/*-----------------------------------------------------------------------------

 2002, Jeff M Soft und Hardware GmbH, All Rights Reserved

-----------------------------------------------------------------------------*/

#include "Module.h"
#include "SEMod_struct.h"
#include "SEPin.h"
#include <assert.h>

// define some constants to make referencing in/outs clearer
#define PN_INPUT1 0
#define PN_INPUT2 1

// wait for waveform to restart (and UI to be updated)
void Module::WaitForTrigger1(long start_pos, long sampleframes)
{
	FSAMPLE *input = input_ptr1 + start_pos;
	for( int s = sampleframes ; s > 0 ; s-- )
	{
		if( *input++ <= 0.f )
		{
			m_idx = 0;
			SET_PROCESS_FUNC( Module::WaitForTrigger2 );
			WaitForTrigger2( start_pos + sampleframes - s, s);
			return;
		}
	}

	timeout -= sampleframes;
	if( timeout < 0 )
	{
		SET_PROCESS_FUNC( Module::WaitForTrigger2 );
	}
}

void Module::WaitForTrigger2(long start_pos, long sampleframes)
{
	FSAMPLE *input = input_ptr1 + start_pos;

	for( int s = sampleframes ; s > 0 ; s-- )
	{
		if( *input++ > 0.f )
		{
			ForceTrigger();

			// simple check that upsteam module is behaving
//			((Ctl_Scope*)GetCUG())->input_state_error_flag = s < sampleframes && GetPlug(0)->getState() == ST_STOP;

			(this->*(getSubProcess()))( start_pos + sampleframes - s, s);
			return;
		}
	}

	timeout -= sampleframes;
	if( timeout < 0 )
	{
		ForceTrigger();
	}
}

void Module::sub_process(long start_pos, long sampleframes)
{
	FSAMPLE *input = input_ptr1 + start_pos;

	int count = SCOPE_BUFFER_SIZE + 2 - m_idx;
	if( count > sampleframes )
		count = sampleframes;

	int remain = sampleframes - count;

	for(int chan = 0 ; chan < CHANNELS; chan++ )
	{
		if( m_channel_active[chan] )
		{
			int i = m_idx;
			for( int c = count ; c > 0 ;c-- )
			{
				m_results[chan][i++] = *input++;
			}
		}
		input = input_ptr2 + start_pos;
	}

	m_idx += count;

	if( m_idx > SCOPE_BUFFER_SIZE + 1 )
	{
		send_result_to_gui();
		(this->*(getSubProcess()))( start_pos + sampleframes - remain, remain);
	}
}

// same as sub process, but don't record samples (because UI redrawing)
void Module::sub_process_cruise(long start_pos, long sampleframes)
{
	int count = SCOPE_BUFFER_SIZE + 2 - m_idx;
	if( count > sampleframes )
		count = sampleframes;

	m_idx += count;

	if( m_idx > SCOPE_BUFFER_SIZE + 1 )
	{
		int remain = sampleframes - count;

		SET_PROCESS_FUNC( Module::WaitForTrigger1 );
		timeout = (int)getSampleRate() / 3;
		(this->*(getSubProcess()))( start_pos + sampleframes - remain, remain);
	}
}

void Module::ForceTrigger()
{
	m_idx = 0;

	if( m_gui_busy == false )
	{
		SET_PROCESS_FUNC( Module::sub_process );
	}
	else
	{
		SET_PROCESS_FUNC( Module::sub_process_cruise );
	}
}

#if defined( _DEBUG )
void Module::TestQue()
{
	return;

	const int sz = 300000;
	unsigned char *test_data = new unsigned char[sz];
	for( int i = 0 ; i < sz ;i++ )
	{
		test_data[i] = i & 0xff;
	}

	SendStringToGui( id_to_int('t','e','s','t'), sz, test_data);

	delete [] test_data;

	RUN_AT(SampleClock() + 20000, &Module::TestQue);
}
#endif

void Module::send_result_to_gui()
{
	if( m_gui_busy == false )
	{
		m_gui_busy = true;
		SendStringToGui( id_to_int('d','a','t','a'), m_results_blob.size, (unsigned char *) m_results_blob.data );
	}

	SET_PROCESS_FUNC( Module::WaitForTrigger1 );
	timeout = (int)getSampleRate() / 3;
}

void Module::OnGuiNotify( int p_user_msg_id, int p_size, void *p_data )
{
	SEModule_base::OnGuiNotify( p_user_msg_id, p_size, p_data );

	if( p_user_msg_id == id_to_int('a','c','k',0) )
	{
		m_gui_busy = false;
	}
}

void Module::SendStringToGui( int p_msg_id, int p_length, unsigned char *p_data )
{
	CallHost(seaudioMasterSendStringToGui, p_length, p_msg_id, p_data );
}

Module::Module(seaudioMasterCallback2 seaudioMaster, void *p_resvd1) : SEModule_base(seaudioMaster, p_resvd1)
,m_gui_busy(false)
,m_idx(0)
{
	m_results_blob.data = m_results;
	m_results_blob.size = sizeof(m_results);
}

Module::~Module()
{
	// This is where you free any memory/resources your module has created
}

void Module::open()
{
	SEModule_base::open();	// always call the base class

	SET_PROCESS_FUNC(Module::sub_process);

	for( int i = 0 ; i < CHANNELS ; i++ )
	{
		m_channel_active[i] = getPin(i)->IsConnected();
		if( !m_channel_active[i] )
		{
			// if a channel is not active, don't want to draw it.
			// so set it's value really big ( so it draws off-screen )
			for( int j = 0 ; j < SCOPE_BUFFER_SIZE ; j++ )
			{
				m_results[i][j] = 1000.f;
			}
		}
	}
#if defined( _DEBUG )
	RUN_AT( 1000, &Module::TestQue );
#endif
}

// describe your module
void Module::getModuleProperties(SEModuleProperties* properties)
{
	// describe the plugin, this is the name the end-user will see.
	properties->name = "Scope2";								// !!TODO!!

	// return a unique string 32 characters max
	// if posible include manufacturer and plugin identity
	// this is used internally by SE to identify the plug.
	// No two plugs may have the same id.
	properties->id = "Synthedit Scope2";						// !!TODO!!

	// Info, may include Author, Web page whatever
	properties->about = "by Jeff M";	
	properties->flags = UGF_POLYPHONIC_AGREGATOR|UGF_VOICE_MON_IGNORE;
	properties->gui_flags = CF_CONTROL_VIEW|CF_STRUCTURE_VIEW;

	properties->sdk_version = SDK_VERSION;
}

// describe the pins (plugs and parameters)
bool Module::getPinProperties (long index, SEPinProperties* properties)
{
	switch( index )								// !!TODO!! list your in / out plugs
	{
		// typical input plug (inputs are listed first)
	case 0:
		properties->name				= "Signal A";
		properties->variable_address	= &input_ptr1;
		properties->direction			= DR_IN;
		properties->datatype			= DT_FSAMPLE;
		properties->default_value		= "0";
		break;
	case 1:
		properties->name				= "Signal B";
		properties->variable_address	= &input_ptr2;
		properties->direction			= DR_IN;
		properties->datatype			= DT_FSAMPLE;
		properties->default_value		= "0";
		break;
	default:
		return false; // host will ask for plugs 0,1,2,3 etc. return false to signal when done
	};

	return true;
}
