#include "guimodule.h"
#include "windows.h"
#include <stdio.h>
#include <assert.h>

#define NODE_SIZE 6
#define TABLE_SIZE 512.f
#define DEFAULT_VALUE "(-5.0,-5.0)(-4.0,-4.0)(-3.0,-3.0)(-2.0,-2.0)(-1.0,-1.0)(0.0,0.0)(1.0,1.0)(2.0,2.0)(3.0,3.0)(4.0,4.0)(5.0,5.0)"

GuiModule::GuiModule(seGuiCallback seaudioMaster, void *p_resvd1) : SEGUI_base(seaudioMaster, p_resvd1)
,drag_node(-1)
,m_inhibit_update(false)
{
	CallHost(seGuiHostSetWindowSize, 100, 100 );
	CallHost(seGuiHostSetWindowSizeable, 1 );

	//testing
//	CallHost( seGuiHostSetWindowType, 1 );
}

void GuiModule::Initialise(bool loaded_from_file)
{
	SEGUI_base::Initialise(loaded_from_file);

	OnValueChanged(); // initial value
}

#define PN_SHAPE 0

void GuiModule::OnGuiPinValueChange(SeGuiPin *p_pin)
{
	OnValueChanged();
}

void GuiModule::OnValueChanged()
{
	if( GetValueS().empty() ) // set a default straight line
	{
		SetValueS( DefaultValue() );
		return; // rely on recursion to re-call this routine
	}

	if( !m_inhibit_update )
	{
		UpdateNodes( nodes, GetValueS() );
		InvalidateControl();
	}
}

SeSdkString GuiModule::GetValueS()
{
	return getPin(PN_SHAPE)->getValueText();
}

void GuiModule::SetValueS(SeSdkString p_string)
{
	getPin(PN_SHAPE)->setValueText(p_string);
}

SeSdkString GuiModule::DefaultValue()
{
	return DEFAULT_VALUE;
}

// convert string value into segment list
void GuiModule::UpdateNodes( mypoint *nodes, const SeSdkString &p_values )
{
	SeSdkString s(p_values);

	// convert CString of numbers to array of screen co-ords
	int i = 0;
	while( s.length() > 0 && i < WS_NODE_COUNT )
	{
		int p1 = s.find("(");
		int p2 = s.find(",");
		int p3 = s.find(")");
		if( p3 < 1 )
			return;

		float x = StringToFloat( s.substr(p1+1,p2-p1-1) );
		float y = StringToFloat( s.substr(p2+1,p3-p2-1) );
		x = ( x + 5.f ) * 10.f; // convert to screen co-ords
		y = ( -y + 5.f ) * 10.f;

		nodes[i].x = x;
		nodes[i++].y = y;

		p3++;
		s = s.substr(p3, s.length() - p3); //.Right( s.length() - p3 - 1);
	}

	if( i == 0 ) // if string empty, set defaults
	{
		UpdateNodes( nodes, DEFAULT_VALUE );
	}
}

float GuiModule::StringToFloat(const SeSdkString &p_string)
{
	char * temp;	// convert string to float
	return (float) strtod( p_string.data(), &temp );
}

void GuiModule::DrawScale( HDC hDC, SEWndInfo *wi )
{
	float v_scale = wi->height / 2.15f;
	float h_scale = wi->width / 2.15f;
	int mid_x = wi->width/2;
	int mid_y = wi->height/2;

	// create a green pen
	HPEN pen = CreatePen( PS_SOLID, 1, RGB(0,100,0) ); // dark green

	// 'select' it
	HGDIOBJ old_pen = SelectObject( hDC, pen );

	// BACKGROUND LINES
	// horizontal line
	MoveToEx(hDC, 0, mid_y, 0 );
	LineTo(hDC, wi->width, mid_y );
	// horiz center line
	MoveToEx(hDC, mid_x, 0, 0 );
	LineTo(hDC, mid_x, wi->height );

	// vertical center line
	MoveToEx(hDC, mid_x,0, 0);
	LineTo(hDC, mid_x, wi->height);

	// diagonal
	MoveToEx(hDC, 0, wi->height, 0);
	LineTo(hDC, wi->width,0);

	int tick_width;
	for( int v = -10 ; v < 11 ; v += 2 )
	{
		float y = v * v_scale / 10.f;
		float x = v * h_scale / 10.f;
		
		if( v % 5 == 0 )
			tick_width = 4;
		else
			tick_width = 2;

		// X-Axis ticks
		MoveToEx(hDC, mid_x - tick_width,mid_y + (int)y, 0 );
		LineTo(hDC, mid_x + tick_width,mid_y + (int)y );

		// Y-Axis ticks
		MoveToEx(hDC,mid_x + (int)x, mid_y - tick_width, 0 );
		LineTo(hDC,mid_x + (int)x, mid_y + tick_width );
	}

	// cleanup
	SelectObject( hDC, old_pen );
	DeleteObject(pen);

	// labels
	if( wi->height > 30 )
	{
		// Set up the font

//		DcFontInfo oldfont(p_child->Skin(),&pDC, _T("tty"));
		int font_height = 10; //p_child->Skin()->getFontDescription(_T("tty"))->AverageCharSize().cy;

		LOGFONT lf;
		memset(&lf, 0, sizeof(LOGFONT));   // Clear out structure.

		strcpy(lf.lfFaceName, "Terminal" );    // face name
		lf.lfHeight = -font_height;

		HFONT font = CreateFontIndirect(&lf);
		HGDIOBJ old_font = SelectObject( hDC, font );

		SetTextColor( hDC, RGB(0,250,0) );
		SetBkMode( hDC, TRANSPARENT );
		SetTextAlign( hDC, TA_LEFT );

		char txt[10];
		// Y-Axis text
		for( float fv = -5 ; fv < 5.1 ; fv += 2.0 )
		{
			float y = fv * v_scale / 5.f;
			if( fv != -1.f )
			{
				sprintf( txt, "%2.0f", fv );
				TextOut( hDC, mid_x + tick_width, mid_y - (int) y - font_height/2, txt, strlen(txt) );
			}
		}

		int orig_ta = SetTextAlign( hDC, TA_CENTER );

		// X-Axis text
		for( float fv = -4 ; fv < 4.1 ; fv += 2.0 )
		{
			float y = fv * h_scale / 5.f;
			if( fv != -1.f )
			{
				sprintf( txt, "%2.0f", fv );
				TextOut( hDC, mid_x + (int)y, mid_y + tick_width, txt, strlen(txt) );
			}
		}

		// cleanup
		SelectObject( hDC, old_font );
		DeleteObject(font);
		SetTextAlign( hDC, orig_ta );
	}
}

void GuiModule::paint( HDC hDC, SEWndInfo *wi )
{

	int ctl_width = wi->width;
	int ctl_height = wi->height;
	float vscale = ctl_height * 0.01f;
	float hscale = ctl_width * 0.01f;
	int mid_x = ctl_width/2;
	int mid_y = ctl_height/2;

	// Fill in solid background black
	HBRUSH background_brush = CreateSolidBrush( RGB(0,0,0) );

	RECT r;
	r.top = r.left = 0;
	r.right = wi->width + 1;
	r.bottom = wi->height + 1;
	FillRect( hDC, &r, background_brush );

	// cleanup objects
    DeleteObject(background_brush);

	// draw scale markings
	DrawScale(hDC,wi);

	// create a green pen
	HPEN pen = CreatePen( PS_SOLID, 1, RGB(0,255,0) ); // light green

	// 'select' it
	HGDIOBJ old_pen = SelectObject( hDC, pen );

	int i = WS_NODE_COUNT - 1;

	MoveToEx(hDC, nodes[i].x * hscale + 0.5f, nodes[i].y * vscale + 0.5f, 0 );
	i--;

	for( ; i >= 0 ; i-- )
	{
		LineTo(hDC, nodes[i].x * hscale + 0.5f, nodes[i].y * vscale + 0.5f);
	}

	// nodes
	sepoint pts[5]; // holds square node points

	for( i = WS_NODE_COUNT - 1; i >= 0 ; i-- )
	{
		int x = nodes[i].x * hscale - NODE_SIZE/2 + 0.5f;
		int y = nodes[i].y * vscale - NODE_SIZE/2 + 0.5f;
		pts[0] = sepoint(x,y);
		pts[1] = sepoint(x + NODE_SIZE,y);
		pts[2] = sepoint(x + NODE_SIZE,y + NODE_SIZE);
		pts[3] = sepoint(x ,y + NODE_SIZE);
		pts[4] = pts[0];
		Polyline( hDC, pts, 5 );
	}

	// display drag node co-ords
		if( drag_node > -1 )
		{
			const int font_height = 10;

			LOGFONT lf;
			memset(&lf, 0, sizeof(LOGFONT));   // Clear out structure.

			strcpy(lf.lfFaceName, "Terminal" );    // face name
			lf.lfHeight = -font_height;

			HFONT font = CreateFontIndirect(&lf);
			HGDIOBJ old_font = SelectObject( hDC, font );

			SetTextColor( hDC, RGB(0,250,0) );
			SetBkMode( hDC, TRANSPARENT );
			SetTextAlign( hDC, TA_LEFT );

			mypoint p(nodes[drag_node]);
			double x = p.x/10.0 - 5.0;
			double y = 5.0 - p.y/10.0;
			char txt[20];
//			text.Format(_T("%3.1f,%3.1f"),x,y);
			sprintf( txt, "%3.1f,%3.1f",x,y );

			TextOut( hDC, 0, 0, txt, strlen(txt) );

			// cleanup
			SelectObject( hDC, old_font );
			DeleteObject(font);
		}

	// cleanup
	SelectObject( hDC, old_pen );
	DeleteObject(pen);
}

void GuiModule::OnLButtonDown( SEWndInfo *wi, UINT nFlags, sepoint point )
{
	/* testing pop up menu...
	// get this module's window handle
	HWND hw = (HWND) CallHost( seGuiHostGetWindowHandle, wi->context_handle );

	RECT r;
	// find it's location (so we know where to draw pop up menu)
	GetWindowRect(hw, &r);

	// create a pop-up menu
	HMENU hm = CreatePopupMenu();

	// add some items to it..
	AppendMenu( hm, MF_STRING , 1, "Cat" );
	AppendMenu( hm, MF_STRING , 2, "Dog" );

	// show the menu
	int selection = TrackPopupMenu( hm, TPM_LEFTALIGN|TPM_NONOTIFY|TPM_RETURNCMD,r.left ,r.top,0, hw, 0);

	// clean up
	DestroyMenu(hm);
	return;
*/
	int ctl_width = wi->width;
	int ctl_height = wi->height;

	float vscale = ctl_height * 0.01f;
	float hscale = ctl_width * 0.01f;

	drag_node = -1;

	for( int i = WS_NODE_COUNT-1; i >= 0 ; i-- )
	{
		mypoint p(nodes[i].x * hscale,nodes[i].y * vscale);
		RECT n;
		n.top = p.y - NODE_SIZE/2;
		n.bottom = p.y + NODE_SIZE/2;
		n.left = p.x - NODE_SIZE/2;
		n.right = p.x + NODE_SIZE/2;
		if( PtInRect(&n,point) )
		{
			drag_node = i;
			m_ptPrev = point;
			SetCapture(wi); // get mouse moves
			break;
		}
	}
}

void GuiModule::OnMouseMove( SEWndInfo *wi, UINT nFlags, sepoint point )
{
	if( !GetCapture(wi) )
		return;

	if( drag_node > -1 )
	{
		int ctl_width = wi->width;
		int ctl_height = wi->height;
		float vscale = ctl_height * 0.01f;
		float hscale = ctl_width * 0.01f;

		float left = 0.f;
		float right = 100.f;

		if( drag_node > 0 )
			left = nodes[drag_node - 1].x + 1;

		if( drag_node < WS_NODE_COUNT - 1 )
			right = nodes[drag_node + 1].x - 1;

		if( drag_node == 0 )
			right = 0.f;

		if( drag_node == WS_NODE_COUNT-1 )
			left = 100.f;

		mypoint p(nodes[drag_node]);
		p.x += (float)(point.x - m_ptPrev.x ) / hscale;
		p.y += (float)(point.y - m_ptPrev.y ) / vscale;

		// constain
		p.x = max(p.x,left);
		p.x = min(p.x,right);
		p.y = max(p.y,0.f);
		p.y = min(p.y,100.f);

		nodes[drag_node] = p;
		m_ptPrev = point;

		char v[300] = "";
		for( int i = 0 ; i < WS_NODE_COUNT; i++ )
		{
			mypoint p(nodes[i]);
			char pt[20];
			double x = p.x/10.0 - 5.0;
			double y = 5.0 - p.y/10.0;
			sprintf(pt,"(%3.1f,%3.1f)",x,y);
			strcat(v,pt);
			assert( strlen(v) < 280 );
		}

		m_inhibit_update = true; // prevent jitter due to float->text->float conversion
		SetValueS( v ); // should default to current patch
		m_inhibit_update = false;
		InvalidateControl();
	}
	
	//TODO
//	SetValueString( CurrentPatch(), v ); // should default to current patch
}

void GuiModule::OnLButtonUp( SEWndInfo *wi, UINT nFlags, sepoint point )
{
	if( !GetCapture(wi) )
		return;

	ReleaseCapture(wi); // don't want further mouse move events
	drag_node = -1;
	// clear on-screen drag co-ords
	InvalidateControl();
}

void GuiModule::OnModuleMsg(int p_user_msg_id,int p_length, void * p_data)
{/*
	assert( p_length == sizeof(values) );
	memcpy(values,p_data,p_length);

	// aknowledge
	int h = Handle();
	char *msg = "ack";
	int size = 3;
////////////// EXPERIMENTAL ////////////
SendStringToAudio( 4, &h );
SendStringToAudio( 4, &size );
SendStringToAudio( size, msg );
///////////////////////////////////////

	InvalidateControl();*/
}

void GuiModule::SendStringToAudio( int p_length, void *p_data )
{
	CallHost(seGuiHostSendStringToAudio, p_length, 0, p_data );
}

int GuiModule::Handle(void)
{
	return CallHost(seGuiHostGetHandle);
}

int GuiModule::InvalidateControl(void)
{
	return CallHost(seGuiHostRequestRepaint);
}



